#!/usr/bin/env python
# -*- encoding: utf-8 -*-
###########################################################################
#    Module Writen to OpenERP, Open Source Management Solution
#
#    Copyright (c) 2019 Grupo SACSA - http://www.gruposacsa.com.mx
#    All Rights Reserved.
############################################################################
#    Coded by:
#       Jorge Alfonso Medina Uriarte (desarrollo.sacsa@gruposacsa.com.mx)
############################################################################
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


from openerp import fields, models, api
import datetime as dt
from openerp.exceptions import Warning as UserError
from datetime import datetime


class cost_policy_report(models.TransientModel):
    _name = 'cost.policy.report'
    _description = 'Cost Policy Report'

    date_last_process = fields.Date(
        'Date Laste Process', default=lambda self: self._get_default_last_date(),
    )
    date_end = fields.Date(
        'Date End', default=lambda self: dt.date.today().strftime('%Y-%m-%d'),
    )
    detail_ids = fields.One2many(
        'cost.policy.report.detail',
        'cost_policy_report_id',
        string='Details',
    )

    @api.model
    def _get_default_last_date(self):
        last_date = self.pool.get('ir.config_parameter').get_param(self.env.cr, self.env.uid,
                                                                   'last.date.update.cost.policy')
        return last_date

    @api.multi
    def update_policy(self):
        # Elimna Registros
        query = """Truncate Table poliza_ids;"""
        self.env.cr.execute(query)

        company_id = 3  # NUTRIVE

        where_filter = """
        WHERE aml.company_id = %s AND aml.date >= '%s'
            AND aml.account_id IN (fn_get_product_expense_account( aml.product_id, aml.company_id), 
            fn_get_product_output_account(product_id, aml.company_id))
            AND aml.product_uom_id <> pt.uom_id AND aj.type = 'sale' 
            AND pt.uos_coeff > 0
            And aml.id not in (select id from poliza_historia_ids)
        ORDER BY aml.date, aml.id;""" % (company_id, self.date_last_process)

        query = """Insert Into poliza_ids
                    SELECT aml.id as id_det, am.id,
                        pp.id as product_id, aj.name journal, am.name poliza, rp.name cliente, 
                        pt.name product, aml.date, aml.ref, aml.debit, aml.credit, aml.quantity, 
                        ROUND( 1 / pt.uos_coeff, 2 ) coef,
                        CASE WHEN debit > 0 THEN round(debit / quantity,2) ELSE round(credit / quantity,2) END costo,
                        ROUND( aml.quantity * (1 / pt.uos_coeff), 2 ) realqty,                                    
                        Round((ROUND((aml.quantity * (1 / pt.uos_coeff)),2)) * (CASE WHEN debit > 0 THEN 
                            round(debit / quantity,2) ELSE round(credit / quantity,2) END),2) costoreal
                    FROM account_move_line aml INNER JOIN product_product pp ON (aml.product_id = pp.id)
                                    INNER JOIN product_template pt ON (pp.product_tmpl_id = pt.id)
                                    INNER JOIN account_account aa ON (aml.account_id = aa.id)
                                    INNER JOIN account_move am ON (aml.move_id = am.id)
                                    INNER JOIN account_journal aj ON (aml.journal_id = aj.id)
                                    INNER JOIN res_partner rp ON (aml.partner_id = rp.id)                    
                   """
        query = query + where_filter
        self.env.cr.execute(query)

        # Inserta Procesadas
        query = """Insert Into poliza_historia_ids                             
                   Select *, '%s'
                   FROM poliza_ids ids;""" % (datetime.now().strftime('%Y-%m-%d %H:%M:%S'))
        self.env.cr.execute(query)

        # actualiza
        query = """UPDATE account_move_line AS aml SET 
                        debit = (CASE WHEN aml.debit > 0 THEN ids.CostoReal ELSE 0 End), 
                        credit = (CASE WHEN aml.credit > 0 THEN ids.CostoReal ELSE 0 End),
                        balance = (CASE WHEN aml.credit > 0 THEN ids.CostoReal * -1 ELSE ids.CostoReal End)
                    FROM poliza_ids ids
                    WHERE ids.id = aml.id;"""
        self.env.cr.execute(query)

        # update last date
        record_ids = self.env['ir.config_parameter'].search([('key', '=', 'last.date.update.cost.policy')])
        for record in record_ids:
            record.write({
                'value': self.date_end
            })

        return True

    @api.multi
    def execute_report(self):
        company_id = 3  # NUTRIVE

        where_filter = """
        WHERE aml.company_id = %s AND aml.date >= '%s'
            AND aml.account_id IN (fn_get_product_expense_account( aml.product_id, aml.company_id), 
                fn_get_product_output_account(product_id, aml.company_id))
            AND aml.product_uom_id <> pt.uom_id AND aj.type = 'sale' 
            AND pt.uos_coeff > 0
            And aml.id not in (select id from poliza_historia_ids)
        ORDER BY aml.date, aml.id;""" % (company_id, self.date_last_process)

        query = """SELECT aml.id as id_det, am.id,
                        pp.id as product_id, aj.name journal, am.name poliza, rp.name cliente, 
                        pt.name product, aml.date, aml.ref, aml.debit, aml.credit, aml.quantity, 
                        ROUND( 1 / pt.uos_coeff, 2 ) coef,
                        CASE WHEN debit > 0 THEN round(debit / quantity,2) ELSE round(credit / quantity,2) END costo,
                        ROUND( aml.quantity * (1 / pt.uos_coeff), 2 ) realqty,                                    
                        Round((ROUND((aml.quantity * (1 / pt.uos_coeff)),2)) * (CASE WHEN debit > 0 THEN 
                            round(debit / quantity,2) ELSE round(credit / quantity,2) END),2) costoreal
                    FROM account_move_line aml INNER JOIN product_product pp ON (aml.product_id = pp.id)
                                    INNER JOIN product_template pt ON (pp.product_tmpl_id = pt.id)
                                    INNER JOIN account_account aa ON (aml.account_id = aa.id)
                                    INNER JOIN account_move am ON (aml.move_id = am.id)
                                    INNER JOIN account_journal aj ON (aml.journal_id = aj.id)
                                    INNER JOIN res_partner rp ON (aml.partner_id = rp.id)                    
                   """
        query = query + where_filter

        self.env.cr.execute(query)
        # get info
        # query = """Select * From poliza_ids;"""
        # self.env.cr.execute(query)
        registros = self.env.cr.fetchall()
        if len(registros) > 0:
            detail_ids = []
            date_format = "%Y-%m-%d"
            for move in registros:
                rs = {
                    'cost_policy_report_id': self.id,
                    'move_line_id': move[0] or 0,
                    'move_id': move[1] or 0,
                    'product_id': move[2] or 0,
                    'journal': move[3] or '',
                    'policy': move[4] or '',
                    'client': move[5] or '',
                    'product': move[6] or '',
                    'move_line_date': datetime.strptime(move[7], date_format),
                    'move_line_ref': move[8] or '',
                    'debit': move[9] or 0,
                    'credit': move[10] or 0,
                    'quantity': move[11] or 0,
                    'coef': move[12] or 0.0,
                    'cost': move[13] or 0.0,
                    'realqty': move[14] or 0.0,
                    'costoreal': move[15] or 0.0
                }
                detail_ids.append(rs)
            self.detail_ids = detail_ids
            return {
                'type': 'ir.actions.report.xml',
                'report_name': 'cost.policy.report.aeroo',
                'datas': {
                    'model': 'cost.policy.report',
                    'id': self.id,
                    'ids': [self.id],
                    'report_type': 'ods'
                },
                'nodestroy': True
            }
        else:
            raise UserError("There are no records.")
